@@ -4,6 +4,8 @@ import subprocess
 
 import pytest
 
+import ecl2df
+
 
 @pytest.mark.integration
 def test_integration():
@@ -21,3 +23,9 @@ def test_integration():
         assert exception.value.returncode == 2
     # ref: https://stackoverflow.com/questions/23714542/  \
     #              why-does-pythons-argparse-use-an-error-code-of-2-for-systemexit
+
+    for submodule in ecl2df.SUBMODULES:
+        helptext = subprocess.check_output(["ecl2csv", submodule, "-h"])
+        # Test that this option is hidden, the argument is only there
+        # to support optional number of arguments in ERT forward models.
+        assert "hiddenemptyplaceholders" not in str(helptext)
