@@ -1,12 +1,13 @@
 """Test module for nnc2df"""
 
-import sys
+import io
 from pathlib import Path
+import subprocess
 
 import pandas as pd
 import pytest
 
-from ecl2df import nnc, faults, ecl2csv, trans
+from ecl2df import nnc, faults, trans, ecl2csv
 from ecl2df.eclfiles import EclFiles
 
 TESTDIR = Path(__file__).absolute().parent
@@ -97,10 +98,10 @@ def test_df2ecl_editnnc(tmpdir):
     print(nnc.df2ecl_editnnc(nnc.df(eclfiles).head(4).assign(TRANM=0.1)))
 
 
-def test_main(tmpdir):
+def test_main(tmpdir, mocker):
     """Test command line interface"""
     tmpcsvfile = tmpdir.join("nnc.csv")
-    sys.argv = ["ecl2csv", "nnc", "-v", DATAFILE, "-o", str(tmpcsvfile)]
+    mocker.patch("sys.argv", ["ecl2csv", "nnc", "-v", DATAFILE, "-o", str(tmpcsvfile)])
     ecl2csv.main()
 
     assert Path(tmpcsvfile).is_file()
@@ -108,3 +109,12 @@ def test_main(tmpdir):
     assert not disk_df.empty
     assert "I1" in disk_df
     assert "TRAN" in disk_df
+
+
+def test_magic_stdout():
+    """Test that we can pipe the output into a dataframe"""
+    result = subprocess.run(
+        ["ecl2csv", "nnc", "-o", "-", DATAFILE], check=True, stdout=subprocess.PIPE
+    )
+    df_stdout = pd.read_csv(io.StringIO(result.stdout.decode()))
+    assert not df_stdout.empty
